@@ -1502,7 +1502,7 @@ markup_entry_get_value (MarkupEntry *entry,
             }
 
           /* Quit as soon as we have the best possible locale */
-          if (local_schemas[0] != NULL)
+          if (local_schemas[0] != NULL && c_local_schema != NULL)
             break;
 
           tmp = tmp->next;
@@ -1522,24 +1522,22 @@ markup_entry_get_value (MarkupEntry *entry,
        * fall back to C locale if we can
        */
 
-      g_assert (best != NULL);
-
-      if (best->locale)
+      if (best && best->locale)
 	gconf_schema_set_locale (schema, best->locale);
       else
 	gconf_schema_set_locale (schema, "C");
 
-      if (best->default_value)
+      if (best && best->default_value)
         gconf_schema_set_default_value (schema, best->default_value);
       else if (c_local_schema && c_local_schema->default_value)
         gconf_schema_set_default_value (schema, c_local_schema->default_value);
 
-      if (best->short_desc)
+      if (best && best->short_desc)
         gconf_schema_set_short_desc (schema, best->short_desc);
       else if (c_local_schema && c_local_schema->short_desc)
         gconf_schema_set_short_desc (schema, c_local_schema->short_desc);
 
-      if (best->long_desc)
+      if (best && best->long_desc)
         gconf_schema_set_long_desc (schema, best->long_desc);
       else if (c_local_schema && c_local_schema->long_desc)
         gconf_schema_set_long_desc (schema, c_local_schema->long_desc);
@@ -3704,6 +3702,10 @@ save_tree (MarkupDir  *dir,
   int new_fd;
   char *filename;
   char *new_filename;
+#ifdef G_OS_WIN32
+  char *tmp_filename;
+  gboolean target_renamed;
+#endif
   char *err_str;
   gboolean write_failed;
   GSList *tmp;
@@ -3716,6 +3718,9 @@ save_tree (MarkupDir  *dir,
   filename = markup_dir_build_file_path (dir, save_as_subtree);
   
   new_filename = g_strconcat (filename, ".new", NULL);
+#ifdef G_OS_WIN32
+  tmp_filename = g_strconcat (filename, ".tmp", NULL);
+#endif
   new_fd = open (new_filename, O_WRONLY | O_CREAT, file_mode);
   if (new_fd < 0)
     {
@@ -3728,7 +3733,11 @@ save_tree (MarkupDir  *dir,
    * if there are no entries in it.
    */
   if (dir->entries == NULL && (!save_as_subtree || dir->subdirs == NULL))
-    goto done_writing;
+    {
+      close (new_fd);
+      new_fd = -1;
+      goto done_writing;
+    }
   
   f = fdopen (new_fd, "w");
   if (f == NULL)
@@ -3812,14 +3821,30 @@ save_tree (MarkupDir  *dir,
       goto out;
     }
   
+#ifdef G_OS_WIN32
+  remove (tmp_filename);
+  target_renamed = (rename (filename, tmp_filename) == 0);
+#endif
+
   if (rename (new_filename, filename) < 0)
     {
       err_str = g_strdup_printf (_("Failed to move temporary file \"%s\" to final location \"%s\": %s"),                                 
                                  new_filename, filename, g_strerror (errno));
+#ifdef G_OS_WIN32
+      if (target_renamed)
+	rename (tmp_filename, filename);
+#endif
       goto out;
     }
+#ifdef G_OS_WIN32
+  if (target_renamed)
+    remove (tmp_filename);
+#endif
   
  out:
+#ifdef G_OS_WIN32
+  g_free (tmp_filename);
+#endif
   g_free (new_filename);
   g_free (filename);
   
